//STL
#include <list>
#include <vector>
#include <iterator>
//STD
#include <errno.h>
#include <string>
#include <iostream>
#include <fstream>
#include <sstream>
#include <cstdlib>
#include <cmath>
#include <cstring>
#include <cmath>
#include <ctime>
//STR
#include "macro.h"
#include "iodna.h"
#include "sca.h"
#include "sht.h"
using namespace std;


std::ostream& display(SEEDCODE word, int length, std::ostream& os){
  char lookup[4] = {'a','t','g','c'};
  
  for(int i=0 ; i < length ; i++){
    int code = (word >> 2 *(length - i)) & 0x3;
    os << lookup[code];
  }
  return os;
}



void tokenize(const string& str, vector<string>& tokens, const string& delimiters = ",;")
{
  // Skip delimiters at beginning.
  string::size_type lastPos = str.find_first_not_of(delimiters, 0);
  // Find first "non-delimiter".
  string::size_type pos     = str.find_first_of(delimiters, lastPos);
  
  while (string::npos != pos || string::npos != lastPos)
    {
      // Found a token, add it to the vector.
      tokens.push_back(str.substr(lastPos, pos - lastPos));
      // Skip delimiters.  Note the "not_of"
      lastPos = str.find_first_not_of(delimiters, pos);
      // Find next "non-delimiter"
      pos = str.find_first_of(delimiters, lastPos);
    }
}



int main(int argc, char ** argv) {

  if (argc < 3) {
    cerr << "" << argv[0] << " \"##-#,#-##\" file1 file2 ...  : at least two arguments are needed" << endl;
    exit(1);
  }
  
  vector<string> seeds = vector<string> (0);
  string seeds_str(argv[1]);

  /* 0) tokenize strings */
  tokenize(seeds_str, seeds);
  
  /* 1) read sequences */
  iodna io = iodna();
  for(int j = 2 ; j < argc ; j++) {
    cerr  << "* Reading sequence \"" << argv[j] << "\" ..." << endl;
    io.addFile(string(argv[j]));
  }

  int feature_start = 0;

  /* 2) seed hash and cound */
  for(int seed_nb = 0 ; seed_nb < (int)seeds.size() ; seed_nb++) {
    cerr  << "* Using seed \"" << seeds[seed_nb] << "\" ..." << endl;
    string seed(seeds[seed_nb]);
    sht sc(seed);
    int bitweight = sc.getBitWeight();
    cerr << " - biweight : " << dec << bitweight << endl;    
    // compute sht table
    sc.add(io);      
    // write partial results
    sc.content(cout,feature_start);
    feature_start += 1 << bitweight;
  }
  cout << endl;
  return 0;
}

